@@ -14,12 +14,17 @@ static const int PORT = 5568;
 E131Component::E131Component() {}
 
 E131Component::~E131Component() {
+#if defined(USE_SOCKET_IMPL_BSD_SOCKETS) || defined(USE_SOCKET_IMPL_LWIP_SOCKETS)
   if (this->socket_) {
     this->socket_->close();
   }
+#elif defined(USE_SOCKET_IMPL_LWIP_TCP)
+  this->udp_.stop();
+#endif
 }
 
 void E131Component::setup() {
+#if defined(USE_SOCKET_IMPL_BSD_SOCKETS) || defined(USE_SOCKET_IMPL_LWIP_SOCKETS)
   this->socket_ = socket::socket_ip(SOCK_DGRAM, IPPROTO_IP);
 
   int enable = 1;
@@ -50,6 +55,13 @@ void E131Component::setup() {
     this->mark_failed();
     return;
   }
+#elif defined(USE_SOCKET_IMPL_LWIP_TCP)
+  if (!this->udp_.begin(PORT)) {
+    ESP_LOGW(TAG, "Cannot bind E1.31 to port %d.", PORT);
+    this->mark_failed();
+    return;
+  }
+#endif
 
   join_igmp_groups_();
 }
@@ -59,19 +71,36 @@ void E131Component::loop() {
   int universe = 0;
   uint8_t buf[1460];
 
+#if defined(USE_SOCKET_IMPL_BSD_SOCKETS) || defined(USE_SOCKET_IMPL_LWIP_SOCKETS)
   ssize_t len = this->socket_->read(buf, sizeof(buf));
   if (len == -1) {
     return;
   }
 
   if (!this->packet_(buf, (size_t) len, universe, packet)) {
-    ESP_LOGV(TAG, "Invalid packet received of size %zd.", len);
+    ESP_LOGV(TAG, "Invalid packet received of size %d.", (int) len);
     return;
   }
 
   if (!this->process_(universe, packet)) {
     ESP_LOGV(TAG, "Ignored packet for %d universe of size %d.", universe, packet.count);
   }
+#elif defined(USE_SOCKET_IMPL_LWIP_TCP)
+  while (auto packet_size = this->udp_.parsePacket()) {
+    auto len = this->udp_.read(buf, sizeof(buf));
+    if (len <= 0)
+      continue;
+
+    if (!this->packet_(buf, (size_t) len, universe, packet)) {
+      ESP_LOGV(TAG, "Invalid packet received of size %d.", (int) len);
+      continue;
+    }
+
+    if (!this->process_(universe, packet)) {
+      ESP_LOGV(TAG, "Ignored packet for %d universe of size %d.", universe, packet.count);
+    }
+  }
+#endif
 }
 
 void E131Component::add_effect(E131AddressableLightEffect *light_effect) {
