@@ -76,50 +76,42 @@ def model_schema(config):
         model.get_default(CONF_MINIMUM_UPDATE_INTERVAL, "1s")
     )
     cv_dimensions = cv.Optional if model.get_default(CONF_WIDTH) else cv.Required
-    return (
-        display.FULL_DISPLAY_SCHEMA.extend(
-            spi.spi_device_schema(
-                cs_pin_required=False,
-                default_mode="MODE0",
-                default_data_rate=model.get_default(CONF_DATA_RATE, 10_000_000),
-            )
-        )
-        .extend(
-            {
-                model.option(pin): pins.gpio_output_pin_schema
-                for pin in (CONF_RESET_PIN, CONF_CS_PIN, CONF_BUSY_PIN)
-            }
-        )
-        .extend(
-            {
-                cv.Optional(CONF_ROTATION, default=0): validate_rotation,
-                cv.Required(CONF_MODEL): cv.one_of(model.name, upper=True),
-                cv.Optional(CONF_UPDATE_INTERVAL, default=cv.UNDEFINED): cv.All(
-                    update_interval, cv.Range(min=minimum_update_interval)
-                ),
-                cv.Optional(CONF_TRANSFORM): cv.Schema(
-                    {
-                        cv.Required(CONF_MIRROR_X): cv.boolean,
-                        cv.Required(CONF_MIRROR_Y): cv.boolean,
-                    }
-                ),
-                cv.Optional(CONF_FULL_UPDATE_EVERY, default=1): cv.int_range(1, 255),
-                model.option(CONF_DC_PIN, fallback=None): pins.gpio_output_pin_schema,
-                cv.GenerateID(): cv.declare_id(class_name),
-                cv.GenerateID(CONF_INIT_SEQUENCE_ID): cv.declare_id(cg.uint8),
-                cv_dimensions(CONF_DIMENSIONS): DIMENSION_SCHEMA,
-                model.option(CONF_ENABLE_PIN): cv.ensure_list(
-                    pins.gpio_output_pin_schema
-                ),
-                model.option(CONF_INIT_SEQUENCE, cv.UNDEFINED): cv.ensure_list(
-                    map_sequence
-                ),
-                model.option(CONF_RESET_DURATION, cv.UNDEFINED): cv.All(
-                    cv.positive_time_period_milliseconds,
-                    cv.Range(max=core.TimePeriod(milliseconds=500)),
-                ),
-            }
+    return display.FULL_DISPLAY_SCHEMA.extend(
+        spi.spi_device_schema(
+            cs_pin_required=False,
+            default_mode="MODE0",
+            default_data_rate=model.get_default(CONF_DATA_RATE, 10_000_000),
         )
+    ).extend(
+        {
+            cv.Optional(CONF_ROTATION, default=0): validate_rotation,
+            cv.Required(CONF_MODEL): cv.one_of(model.name, upper=True),
+            cv.Optional(CONF_UPDATE_INTERVAL, default=cv.UNDEFINED): cv.All(
+                update_interval, cv.Range(min=minimum_update_interval)
+            ),
+            cv.Optional(CONF_TRANSFORM): cv.Schema(
+                {
+                    cv.Required(CONF_MIRROR_X): cv.boolean,
+                    cv.Required(CONF_MIRROR_Y): cv.boolean,
+                }
+            ),
+            cv.Optional(CONF_FULL_UPDATE_EVERY, default=1): cv.int_range(1, 255),
+            model.option(CONF_BUSY_PIN): pins.gpio_input_pin_schema,
+            model.option(CONF_CS_PIN): pins.gpio_output_pin_schema,
+            model.option(CONF_DC_PIN, fallback=None): pins.gpio_output_pin_schema,
+            model.option(CONF_RESET_PIN): pins.gpio_output_pin_schema,
+            cv.GenerateID(): cv.declare_id(class_name),
+            cv.GenerateID(CONF_INIT_SEQUENCE_ID): cv.declare_id(cg.uint8),
+            cv_dimensions(CONF_DIMENSIONS): DIMENSION_SCHEMA,
+            model.option(CONF_ENABLE_PIN): cv.ensure_list(pins.gpio_output_pin_schema),
+            model.option(CONF_INIT_SEQUENCE, cv.UNDEFINED): cv.ensure_list(
+                map_sequence
+            ),
+            model.option(CONF_RESET_DURATION, cv.UNDEFINED): cv.All(
+                cv.positive_time_period_milliseconds,
+                cv.Range(max=core.TimePeriod(milliseconds=500)),
+            ),
+        }
     )
 
 
