@@ -6,7 +6,7 @@ namespace esphome::epaper_spi {
 /**
  * An epaper display that needs LUTs to be sent to it.
  */
-class EpaperWaveshare : public EPaperMono {
+class EpaperWaveshare final : public EPaperMono {
  public:
   EpaperWaveshare(const char *name, uint16_t width, uint16_t height, const uint8_t *init_sequence,
                   size_t init_sequence_length, const uint8_t *lut, size_t lut_length, const uint8_t *partial_lut,
@@ -18,7 +18,7 @@ class EpaperWaveshare : public EPaperMono {
         partial_lut_length_(partial_lut_length) {}
 
  protected:
-  void initialise(bool partial) override;
+  bool initialise(bool partial) override;
   void set_window() override;
   void refresh_screen(bool partial) override;
   void deep_sleep() override;
