@@ -1,9 +1,12 @@
 import logging
 
 import esphome.config_validation as cv
-from esphome.const import CONF_INPUT, CONF_MODE, CONF_NUMBER
+from esphome.const import CONF_INPUT, CONF_MODE, CONF_NUMBER, CONF_SCL, CONF_SDA
 from esphome.pins import check_strapping_pin
 
+# https://github.com/espressif/esp-idf/blob/master/components/esp_hal_i2c/esp32c6/include/hal/i2c_ll.h
+_ESP32C6_I2C_LP_PINS = {"SDA": 6, "SCL": 7}
+
 _ESP32C6_SPI_PSRAM_PINS = {
     24: "SPICS0",
     25: "SPIQ",
@@ -43,3 +46,13 @@ def esp32_c6_validate_supports(value):
 
     check_strapping_pin(value, _ESP32C6_STRAPPING_PINS, _LOGGER)
     return value
+
+
+def esp32_c6_validate_lp_i2c(value):
+    lp_sda_pin = _ESP32C6_I2C_LP_PINS["SDA"]
+    lp_scl_pin = _ESP32C6_I2C_LP_PINS["SCL"]
+    if int(value[CONF_SDA]) != lp_sda_pin or int(value[CONF_SCL]) != lp_scl_pin:
+        raise cv.Invalid(
+            f"Low power i2c interface is only supported on GPIO{lp_sda_pin} SDA and GPIO{lp_scl_pin} SCL for ESP32-C6"
+        )
+    return value
