@@ -18,7 +18,7 @@ namespace esp32_camera_web_server {
 
 enum Mode { STREAM, SNAPSHOT };
 
-class CameraWebServer : public Component {
+class CameraWebServer : public Component, public camera::CameraListener {
  public:
   CameraWebServer();
   ~CameraWebServer();
@@ -31,6 +31,9 @@ class CameraWebServer : public Component {
   void set_mode(Mode mode) { this->mode_ = mode; }
   void loop() override;
 
+  /// CameraListener interface
+  void on_camera_image(const std::shared_ptr<camera::CameraImage> &image) override;
+
  protected:
   std::shared_ptr<camera::CameraImage> wait_for_image_();
   esp_err_t handler_(struct httpd_req *req);
