@@ -1,11 +1,7 @@
 import esphome.codegen as cg
-import esphome.config_validation as cv
 from esphome.components import text_sensor
-from esphome.const import (
-    ENTITY_CATEGORY_NONE,
-    ENTITY_CATEGORY_DIAGNOSTIC,
-    CONF_ID,
-)
+import esphome.config_validation as cv
+from esphome.const import CONF_ID, ENTITY_CATEGORY_DIAGNOSTIC, ENTITY_CATEGORY_NONE
 
 from . import EzoPMP
 
