@@ -107,7 +107,7 @@ CONFIG_SCHEMA = cv.All(
             cv.Required(CONF_MAX_TEMPERATURE): cv.temperature,
         }
     ),
-    cv.only_with_arduino,
+    cv.Any(cv.only_with_arduino, cv.only_on_esp32),
 )
 
 
@@ -126,6 +126,6 @@ async def to_code(config):
     cg.add(var.set_max_temperature(config[CONF_MAX_TEMPERATURE]))
     cg.add(var.set_min_temperature(config[CONF_MIN_TEMPERATURE]))
 
-    cg.add_library("tonia/HeatpumpIR", "1.0.37")
+    cg.add_library("tonia/HeatpumpIR", "1.0.40")
     if CORE.is_libretiny or CORE.is_esp32:
         CORE.add_platformio_option("lib_ignore", ["IRremoteESP8266"])
