@@ -2,8 +2,7 @@
 #include "esphome/core/helpers.h"
 #include "esphome/core/log.h"
 
-namespace esphome {
-namespace ld2420 {
+namespace esphome::ld2420 {
 
 static const char *const TAG = "ld2420.select";
 
@@ -12,5 +11,4 @@ void LD2420Select::control(size_t index) {
   this->parent_->set_operating_mode(this->option_at(index));
 }
 
-}  // namespace ld2420
-}  // namespace esphome
+}  // namespace esphome::ld2420
