@@ -3,7 +3,7 @@
 
 namespace esphome::logger {
 
-void HOT Logger::write_msg_(const char *msg, size_t len) {
+void HOT Logger::write_msg_(const char *msg, uint16_t len) {
   static constexpr size_t TIMESTAMP_LEN = 10;  // "[HH:MM:SS]"
   // tx_buffer_size_ defaults to 512, so 768 covers default + headroom
   char buffer[TIMESTAMP_LEN + 768];
@@ -15,7 +15,7 @@ void HOT Logger::write_msg_(const char *msg, size_t len) {
   size_t pos = strftime(buffer, TIMESTAMP_LEN + 1, "[%H:%M:%S]", &timeinfo);
 
   // Copy message (with newline already included by caller)
-  size_t copy_len = std::min(len, sizeof(buffer) - pos);
+  size_t copy_len = std::min(static_cast<size_t>(len), sizeof(buffer) - pos);
   memcpy(buffer + pos, msg, copy_len);
   pos += copy_len;
 
