@@ -272,9 +272,7 @@ async def obj_hide_to_code(config, action_id, template_arg, args):
     async def do_hide(widget: Widget):
         widget.add_flag("LV_OBJ_FLAG_HIDDEN")
 
-    widgets = [
-        widget.outer if widget.outer else widget for widget in await get_widgets(config)
-    ]
+    widgets = [widget.outer or widget for widget in await get_widgets(config)]
     return await action_to_code(widgets, do_hide, action_id, template_arg, args)
 
 
@@ -285,9 +283,7 @@ async def obj_show_to_code(config, action_id, template_arg, args):
         if widget.move_to_foreground:
             lv_obj.move_foreground(widget.obj)
 
-    widgets = [
-        widget.outer if widget.outer else widget for widget in await get_widgets(config)
-    ]
+    widgets = [widget.outer or widget for widget in await get_widgets(config)]
     return await action_to_code(widgets, do_show, action_id, template_arg, args)
 
 
