@@ -7,12 +7,12 @@ namespace mcp3008 {
 
 static const char *const TAG = "mcp3008.sensor";
 
-float MCP3008Sensor::get_setup_priority() const { return setup_priority::DATA; }
-
 void MCP3008Sensor::dump_config() {
-  ESP_LOGCONFIG(TAG, "MCP3008Sensor:");
-  ESP_LOGCONFIG(TAG, "  Pin: %u", this->pin_);
-  ESP_LOGCONFIG(TAG, "  Reference Voltage: %.2fV", this->reference_voltage_);
+  ESP_LOGCONFIG(TAG,
+                "MCP3008Sensor:\n"
+                "  Pin: %u\n"
+                "  Reference Voltage: %.2fV",
+                this->pin_, this->reference_voltage_);
 }
 
 float MCP3008Sensor::sample() {
