@@ -4,7 +4,10 @@ from esphome import automation
 from esphome.automation import Condition
 import esphome.codegen as cg
 from esphome.components import logger, socket
-from esphome.components.esp32 import add_idf_sdkconfig_option
+from esphome.components.esp32 import (
+    add_idf_sdkconfig_option,
+    include_builtin_idf_component,
+)
 from esphome.config_helpers import filter_source_files_from_platform
 import esphome.config_validation as cv
 from esphome.const import (
@@ -360,6 +363,8 @@ async def to_code(config):
     # This enables low-latency MQTT event processing instead of waiting for select() timeout
     if CORE.is_esp32:
         socket.require_wake_loop_threadsafe()
+        # Re-enable ESP-IDF's mqtt component (excluded by default to save compile time)
+        include_builtin_idf_component("mqtt")
 
     cg.add_define("USE_MQTT")
     cg.add_global(mqtt_ns.using)
