@@ -8,8 +8,7 @@
 #include "esphome/components/switch/switch.h"
 #include "mqtt_component.h"
 
-namespace esphome {
-namespace mqtt {
+namespace esphome::mqtt {
 
 class MQTTSwitchComponent : public mqtt::MQTTComponent {
  public:
@@ -28,14 +27,13 @@ class MQTTSwitchComponent : public mqtt::MQTTComponent {
 
  protected:
   /// "switch" component type.
-  std::string component_type() const override;
+  const char *component_type() const override;
   const EntityBase *get_entity() const override;
 
   switch_::Switch *switch_;
 };
 
-}  // namespace mqtt
-}  // namespace esphome
+}  // namespace esphome::mqtt
 
 #endif
 #endif  // USE_MQTT
