@@ -17,6 +17,8 @@ class OneWireDevice {
   /// @param address of the device
   void set_address(uint64_t address) { this->address_ = address; }
 
+  void set_index(uint8_t index) { this->index_ = index; }
+
   /// @brief store the pointer to the OneWireBus to use
   /// @param bus pointer to the OneWireBus object
   void set_one_wire_bus(OneWireBus *bus) { this->bus_ = bus; }
@@ -25,13 +27,16 @@ class OneWireDevice {
   const std::string &get_address_name();
 
  protected:
+  static constexpr uint8_t INDEX_NOT_SET = 255;
+
   uint64_t address_{0};
+  uint8_t index_{INDEX_NOT_SET};
   OneWireBus *bus_{nullptr};  ///< pointer to OneWireBus instance
   std::string address_name_;
 
   /// @brief find an address if necessary
   /// should be called from setup
-  bool check_address_();
+  bool check_address_or_index_();
 
   /// @brief send command on the bus
   /// @param cmd command to send
