@@ -31,17 +31,18 @@ void Select::publish_state(size_t index) {
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   this->state = option;  // Update deprecated member for backward compatibility
 #pragma GCC diagnostic pop
-  ESP_LOGD(TAG, "'%s': Sending state %s (index %zu)", this->get_name().c_str(), option, index);
-  // Callback signature requires std::string, create temporary for compatibility
-  this->state_callback_.call(std::string(option), index);
+  ESP_LOGD(TAG, "'%s' >> %s (%zu)", this->get_name().c_str(), option, index);
+  this->state_callback_.call(index);
 #if defined(USE_SELECT) && defined(USE_CONTROLLER_REGISTRY)
   ControllerRegistry::notify_select_update(this);
 #endif
 }
 
-const char *Select::current_option() const { return this->has_state() ? this->option_at(this->active_index_) : ""; }
+StringRef Select::current_option() const {
+  return this->has_state() ? StringRef(this->option_at(this->active_index_)) : StringRef();
+}
 
-void Select::add_on_state_callback(std::function<void(std::string, size_t)> &&callback) {
+void Select::add_on_state_callback(std::function<void(size_t)> &&callback) {
   this->state_callback_.add(std::move(callback));
 }
 
