@@ -22,8 +22,10 @@ void SX1509FloatOutputChannel::setup() {
 }
 
 void SX1509FloatOutputChannel::dump_config() {
-  ESP_LOGCONFIG(TAG, "SX1509 PWM:");
-  ESP_LOGCONFIG(TAG, "  sx1509 pin: %d", this->pin_);
+  ESP_LOGCONFIG(TAG,
+                "SX1509 PWM:\n"
+                "  sx1509 pin: %d",
+                this->pin_);
   LOG_FLOAT_OUTPUT(this);
 }
 
