@@ -7,8 +7,7 @@ using namespace esphome::lock;
 
 static const char *const TAG = "template.lock";
 
-TemplateLock::TemplateLock()
-    : lock_trigger_(new Trigger<>()), unlock_trigger_(new Trigger<>()), open_trigger_(new Trigger<>()) {}
+TemplateLock::TemplateLock() = default;
 
 void TemplateLock::setup() {
   if (!this->f_.has_value())
@@ -28,11 +27,11 @@ void TemplateLock::control(const lock::LockCall &call) {
 
   auto state = *call.get_state();
   if (state == LOCK_STATE_LOCKED) {
-    this->prev_trigger_ = this->lock_trigger_;
-    this->lock_trigger_->trigger();
+    this->prev_trigger_ = &this->lock_trigger_;
+    this->lock_trigger_.trigger();
   } else if (state == LOCK_STATE_UNLOCKED) {
-    this->prev_trigger_ = this->unlock_trigger_;
-    this->unlock_trigger_->trigger();
+    this->prev_trigger_ = &this->unlock_trigger_;
+    this->unlock_trigger_.trigger();
   }
 
   if (this->optimistic_)
@@ -42,14 +41,11 @@ void TemplateLock::open_latch() {
   if (this->prev_trigger_ != nullptr) {
     this->prev_trigger_->stop_action();
   }
-  this->prev_trigger_ = this->open_trigger_;
-  this->open_trigger_->trigger();
+  this->prev_trigger_ = &this->open_trigger_;
+  this->open_trigger_.trigger();
 }
 void TemplateLock::set_optimistic(bool optimistic) { this->optimistic_ = optimistic; }
 float TemplateLock::get_setup_priority() const { return setup_priority::HARDWARE; }
-Trigger<> *TemplateLock::get_lock_trigger() const { return this->lock_trigger_; }
-Trigger<> *TemplateLock::get_unlock_trigger() const { return this->unlock_trigger_; }
-Trigger<> *TemplateLock::get_open_trigger() const { return this->open_trigger_; }
 void TemplateLock::dump_config() {
   LOG_LOCK("", "Template Lock", this);
   ESP_LOGCONFIG(TAG, "  Optimistic: %s", YESNO(this->optimistic_));
