@@ -2,6 +2,7 @@
 
 #include "esphome/core/defines.h"
 #ifdef USE_NETWORK
+#include "esphome/core/helpers.h"
 #include "esphome/components/network/ip_address.h"
 #if defined(USE_SOCKET_IMPL_BSD_SOCKETS) || defined(USE_SOCKET_IMPL_LWIP_SOCKETS)
 #include "esphome/components/socket/socket.h"
@@ -9,21 +10,25 @@
 #ifdef USE_SOCKET_IMPL_LWIP_TCP
 #include <WiFiUdp.h>
 #endif
+#include <array>
+#include <initializer_list>
+#include <span>
 #include <vector>
 
-namespace esphome {
-namespace udp {
+namespace esphome::udp {
 
 static const size_t MAX_PACKET_SIZE = 508;
 class UDPComponent : public Component {
  public:
-  void add_address(const char *addr) { this->addresses_.emplace_back(addr); }
+  void set_addresses(std::initializer_list<const char *> addresses) { this->addresses_ = addresses; }
+  /// Prevent accidental use of std::string which would dangle
+  void set_addresses(std::initializer_list<std::string> addresses) = delete;
   void set_listen_address(const char *listen_addr) { this->listen_address_ = network::IPAddress(listen_addr); }
   void set_listen_port(uint16_t port) { this->listen_port_ = port; }
   void set_broadcast_port(uint16_t port) { this->broadcast_port_ = port; }
   void set_should_broadcast() { this->should_broadcast_ = true; }
   void set_should_listen() { this->should_listen_ = true; }
-  void add_listener(std::function<void(std::vector<uint8_t> &)> &&listener) {
+  void add_listener(std::function<void(std::span<const uint8_t>)> &&listener) {
     this->packet_listeners_.add(std::move(listener));
   }
   void setup() override;
@@ -38,7 +43,7 @@ class UDPComponent : public Component {
   uint16_t broadcast_port_{};
   bool should_broadcast_{};
   bool should_listen_{};
-  CallbackManager<void(std::vector<uint8_t> &)> packet_listeners_{};
+  CallbackManager<void(std::span<const uint8_t>)> packet_listeners_{};
 
 #if defined(USE_SOCKET_IMPL_BSD_SOCKETS) || defined(USE_SOCKET_IMPL_LWIP_SOCKETS)
   std::unique_ptr<socket::Socket> broadcast_socket_ = nullptr;
@@ -49,11 +54,10 @@ class UDPComponent : public Component {
   std::vector<IPAddress> ipaddrs_{};
   WiFiUDP udp_client_{};
 #endif
-  std::vector<std::string> addresses_{};
+  FixedVector<const char *> addresses_{};
 
   optional<network::IPAddress> listen_address_{};
 };
 
-}  // namespace udp
-}  // namespace esphome
+}  // namespace esphome::udp
 #endif
