@@ -6,6 +6,7 @@
 #include "string_ref.h"
 #include "helpers.h"
 #include "log.h"
+#include "preferences.h"
 
 #ifdef USE_DEVICES
 #include "device.h"
@@ -138,7 +139,12 @@ class EntityBase {
    * from previous versions, so existing single-device configurations will continue to work.
    *
    * @return uint32_t The unique hash for preferences, including device_id if available.
+   * @deprecated Use make_entity_preference<T>() instead, or preferences won't be migrated.
+   * See https://github.com/esphome/backlog/issues/85
    */
+  ESPDEPRECATED("Use make_entity_preference<T>() instead, or preferences won't be migrated. "
+                "See https://github.com/esphome/backlog/issues/85. Will be removed in 2027.1.0.",
+                "2026.7.0")
   uint32_t get_preference_hash() {
 #ifdef USE_DEVICES
     // Combine object_id_hash with device_id to ensure uniqueness across devices
@@ -151,7 +157,19 @@ class EntityBase {
 #endif
   }
 
+  /// Create a preference object for storing this entity's state/settings.
+  /// @tparam T The type of data to store (must be trivially copyable)
+  /// @param version Optional version hash XORed with preference key (change when struct layout changes)
+  template<typename T> ESPPreferenceObject make_entity_preference(uint32_t version = 0) {
+    static_assert(std::is_trivially_copyable<T>::value, "T must be trivially copyable");
+    return this->make_entity_preference_(sizeof(T), version);
+  }
+
  protected:
+  /// Non-template helper for make_entity_preference() to avoid code bloat.
+  /// When preference hash algorithm changes, migration logic goes here.
+  ESPPreferenceObject make_entity_preference_(size_t size, uint32_t version);
+
   void calc_object_id_();
 
   StringRef name_;
@@ -212,6 +230,16 @@ class EntityBase_UnitOfMeasurement {  // NOLINT(readability-identifier-naming)
   const char *unit_of_measurement_{nullptr};  ///< Unit of measurement override
 };
 
+/// Log entity icon if set (for use in dump_config)
+#define LOG_ENTITY_ICON(tag, prefix, obj) log_entity_icon(tag, prefix, obj)
+void log_entity_icon(const char *tag, const char *prefix, const EntityBase &obj);
+/// Log entity device class if set (for use in dump_config)
+#define LOG_ENTITY_DEVICE_CLASS(tag, prefix, obj) log_entity_device_class(tag, prefix, obj)
+void log_entity_device_class(const char *tag, const char *prefix, const EntityBase_DeviceClass &obj);
+/// Log entity unit of measurement if set (for use in dump_config)
+#define LOG_ENTITY_UNIT_OF_MEASUREMENT(tag, prefix, obj) log_entity_unit_of_measurement(tag, prefix, obj)
+void log_entity_unit_of_measurement(const char *tag, const char *prefix, const EntityBase_UnitOfMeasurement &obj);
+
 /**
  * An entity that has a state.
  * @tparam T The type of the state
