@@ -3,11 +3,16 @@ from __future__ import annotations
 import asyncio
 from contextlib import suppress
 from ipaddress import ip_address
+import logging
 
 from icmplib import NameLookupError, async_resolve
 
 RESOLVE_TIMEOUT = 3.0
 
+_LOGGER = logging.getLogger(__name__)
+
+_RESOLVE_EXCEPTIONS = (TimeoutError, NameLookupError, UnicodeError)
+
 
 async def _async_resolve_wrapper(hostname: str) -> list[str] | Exception:
     """Wrap the icmplib async_resolve function."""
@@ -16,7 +21,21 @@ async def _async_resolve_wrapper(hostname: str) -> list[str] | Exception:
     try:
         async with asyncio.timeout(RESOLVE_TIMEOUT):
             return await async_resolve(hostname)
-    except (TimeoutError, NameLookupError, UnicodeError) as ex:
+    except _RESOLVE_EXCEPTIONS as ex:
+        # If the hostname ends with .local and resolution failed,
+        # try the bare hostname as a fallback since mDNS may not be
+        # working on the system but unicast DNS might resolve it
+        if hostname.endswith(".local"):
+            bare_hostname = hostname[:-6]  # Remove ".local"
+            try:
+                async with asyncio.timeout(RESOLVE_TIMEOUT):
+                    result = await async_resolve(bare_hostname)
+                    _LOGGER.debug(
+                        "Bare hostname %s resolved to %s", bare_hostname, result
+                    )
+                    return result
+            except _RESOLVE_EXCEPTIONS:
+                _LOGGER.debug("Bare hostname %s also failed to resolve", bare_hostname)
         return ex
 
 
